@@ -596,6 +596,16 @@ static void cleanup_header_done_callback(void *context)
     TOK822 *token;
     time_t  tv;
 
+    /*
+     * XXX Workaround: when we reach the end of headers, mime_state_update()
+     * may execute up to three call-backs before returning to the caller:
+     * head_out(), head_end(), and body_out() or body_end(). As long as
+     * call-backs don't return a result, each call-back has to check for
+     * itself if the previous call-back experienced a problem.
+     */
+    if (CLEANUP_OUT_OK(state) == 0)
+	return;
+
     /*
      * Add a missing (Resent-)Message-Id: header. The message ID gives the
      * time in GMT units, plus the local queue ID.
@@ -695,6 +705,7 @@ static void cleanup_header_done_callback(void *context)
 	cleanup_out_format(state, REC_TYPE_PTR, REC_TYPE_PTR_FORMAT, 0L);
 	if ((state->append_hdr_pt_target = vstream_ftell(state->dst)) < 0)
 	    msg_fatal("%s: vstream_ftell %s: %m", myname, cleanup_path);
+	state->body_offset = state->append_hdr_pt_target;
     }
 }
 
@@ -706,6 +717,16 @@ static void cleanup_body_callback(void *context, int type,
 {
     CLEANUP_STATE *state = (CLEANUP_STATE *) context;
 
+    /*
+     * XXX Workaround: when we reach the end of headers, mime_state_update()
+     * may execute up to three call-backs before returning to the caller:
+     * head_out(), head_end(), and body_out() or body_end(). As long as
+     * call-backs don't return a result, each call-back has to check for
+     * itself if the previous call-back experienced a problem.
+     */
+    if (CLEANUP_OUT_OK(state) == 0)
+	return;
+
     /*
      * Crude message body content filter for emergencies. This code has
      * several problems: it sees one line at a time; it looks at long lines
@@ -797,10 +818,10 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
      * current file position so we can compute the message size lateron.
      */
     else if (type == REC_TYPE_XTRA) {
+	state->mime_errs = mime_state_update(state->mime_state, type, buf, len);
 	if (state->milters || cleanup_milters)
 	    /* Make room for body modification. */
 	    cleanup_out_format(state, REC_TYPE_PTR, REC_TYPE_PTR_FORMAT, 0L);
-	state->mime_errs = mime_state_update(state->mime_state, type, buf, len);
 	/* Ignore header truncation after primary message headers. */
 	state->mime_errs &= ~MIME_ERR_TRUNC_HEADER;
 	if (state->mime_errs && state->reason == 0) {
@@ -811,6 +832,7 @@ static void cleanup_message_headerbody(CLEANUP_STATE *state, int type,
 	state->mime_state = mime_state_free(state->mime_state);
 	if ((state->xtra_offset = vstream_ftell(state->dst)) < 0)
 	    msg_fatal("%s: vstream_ftell %s: %m", myname, cleanup_path);
+	state->cont_length = state->xtra_offset - state->data_offset;
 	state->action = cleanup_extracted;
     }
 
@@ -892,6 +914,15 @@ void    cleanup_message(CLEANUP_STATE *state, int type, const char *buf, ssize_t
 					 cleanup_mime_error_callback,
 					 (void *) state);
 
+    /*
+     * XXX Workaround: truncate a long message header so that we don't exceed
+     * the Milter request size limit of 65535.
+     */
+#define KLUDGE_HEADER_LIMIT	60000
+    if ((cleanup_milters || state->milters)
+	&& var_header_limit > KLUDGE_HEADER_LIMIT)
+	var_header_limit = KLUDGE_HEADER_LIMIT;
+
     /*
      * Pass control to the header processing routine.
      */
