@@ -143,12 +143,6 @@
 #define SMFIR_REPLYCODE		'y'	/* reply code etc */
 #define SMFIR_QUARANTINE	'q'	/* quarantine */
 
- /* Introduced with Sendmail 8.14. */
-#define SMFIR_ADDRCPT_PAR	'2'	/* add recipient (incl. ESMTP args) */
-#define SMFIR_CHGFROM		'e'	/* change envelope sender (from) */
-#define SMFIR_SETSYMLIST	'l'	/* set list of symbols (macros) */
-#define SMFIR_SKIP		's'	/* skip */
-
  /*
   * Commands that the filter does not want to receive, and replies that the
   * filter will not send.
@@ -160,23 +154,10 @@
 #define SMFIP_NOBODY		(1L<<4)	/* MTA should not send body */
 #define SMFIP_NOHDRS		(1L<<5)	/* MTA should not send headers */
 #define SMFIP_NOEOH		(1L<<6)	/* MTA should not send EOH */
-#define SMFIP_NR_HDR		(1L<<7)	/* filter will not reply per header */
-#define SMFIP_NOHREPL		SMFIP_NR_HDR
+#define SMFIP_NOHREPL		(1L<<7)	/* filter will not reply per header */
 #define SMFIP_NOUNKNOWN 	(1L<<8)	/* MTA should not send unknown cmd */
 #define SMFIP_NODATA		(1L<<9)	/* MTA should not send DATA */
 
- /* Introduced with Sendmail 8.14. */
-#define SMFIP_SKIP	0x00000400L	/* MTA understands SMFIS_SKIP */
-#define SMFIP_RCPT_REJ	0x00000800L	/* MTA should send rejected RCPTs */
-#define SMFIP_NR_CONN	0x00001000L	/* No reply for connect */
-#define SMFIP_NR_HELO	0x00002000L	/* No reply for HELO */
-#define SMFIP_NR_MAIL	0x00004000L	/* No reply for MAIL */
-#define SMFIP_NR_RCPT	0x00008000L	/* No reply for RCPT */
-#define SMFIP_NR_DATA	0x00010000L	/* No reply for DATA */
-#define SMFIP_NR_UNKN	0x00020000L	/* No reply for UNKN */
-#define SMFIP_NR_EOH	0x00040000L	/* No reply for eoh */
-#define SMFIP_NR_BODY	0x00080000L	/* No reply for body chunk */
-
  /*
   * Modifications that the filter may request at the end of the message body.
   */
@@ -215,6 +196,7 @@
 
 /* Global library. */
 
+#include <mail_params.h>		/* var_line_limit */
 #include <mail_proto.h>
 #include <rec_type.h>
 #include <record.h>
@@ -589,10 +571,10 @@ static int vmilter8_read_data(MILTER8 *milter, ssize_t data_len, va_list ap)
 	    break;
 
 	    /*
-	     * Raw on-the-wire format.
+	     * Raw on-the-wire format, without explicit null terminator.
 	     */
 	case MILTER8_DATA_BUFFER:
-	    if (data_left < 1) {
+	    if (data_left < 0) {
 		msg_warn("milter %s: no data in input packet", milter->m.name);
 		return (milter8_comm_error(milter));
 	    }
@@ -885,9 +867,12 @@ static const char *milter8_event(MILTER8 *milter, int event,
     ssize_t data_size;
     const char *smfic_name;
     const char *smfir_name;
-    MILTERS *parent;
+    MILTERS *parent = milter->m.parent;
     UINT32_TYPE index;
-    const char *edit_resp;
+    const char *edit_resp = 0;
+    const char *retval = 0;
+    VSTRING *body_line_buf = 0;
+    int     done = 0;
 
 #define DONT_SKIP_REPLY	0
 
@@ -989,17 +974,30 @@ static const char *milter8_event(MILTER8 *milter, int event,
     /*
      * Receive the reply or replies.
      * 
+     * Intercept all loop exits so that we can do post body replacement
+     * processing.
+     * 
      * XXX Bound the loop iteration count.
      */
 #define IN_CONNECT_EVENT(e) ((e) == SMFIC_CONNECT || (e) == SMFIC_HELO)
 
-    for (;;) {
+    /*
+     * XXX Don't evaluate this macro's argument multiple times. Since we use
+     * "continue" the macro can't be enclosed in do .. while (0).
+     */
+#define MILTER8_EVENT_BREAK(s) { \
+	retval = (s); \
+	done = 1; \
+	continue; \
+    }
+
+    while (done == 0) {
 	char   *cp;
 	char   *rp;
 	char    ch;
 
 	if (milter8_read_resp(milter, event, &cmd, &data_size) != 0)
-	    return (milter->def_reply);
+	    MILTER8_EVENT_BREAK(milter->def_reply);
 	if (msg_verbose)
 	    msg_info("reply: %s data %ld bytes",
 		     (smfir_name = str_name_code(smfir_table, cmd)) != 0 ?
@@ -1020,7 +1018,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 	case SMFIR_CONTINUE:
 	    if (data_size != 0)
 		break;
-	    return (milter->def_reply);
+	    MILTER8_EVENT_BREAK(milter->def_reply);
 
 	    /*
 	     * Decision: accept this message, or accept all further commands
@@ -1040,7 +1038,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		/* No more events for this message. */
 		milter->state = MILTER8_STAT_ACCEPT_MSG;
 	    }
-	    return (milter->def_reply);
+	    MILTER8_EVENT_BREAK(milter->def_reply);
 
 	    /*
 	     * Decision: accept and silently discard this message. According
@@ -1055,11 +1053,11 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		msg_warn("milter %s: DISCARD action is not allowed "
 			 "for connect or helo", milter->m.name);
 		milter8_conf_error(milter);
-		return (milter->def_reply);
+		MILTER8_EVENT_BREAK(milter->def_reply);
 	    } else {
 		/* No more events for this message. */
 		milter->state = MILTER8_STAT_ACCEPT_MSG;
-		return ("D");
+		MILTER8_EVENT_BREAK("D");
 	    }
 
 	    /*
@@ -1074,9 +1072,9 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		milter8_close_stream(milter);
 #endif
 		milter->state = MILTER8_STAT_REJECT_CON;
-		return (milter8_def_reply(milter, "550 5.7.1 Command rejected"));
+		MILTER8_EVENT_BREAK(milter8_def_reply(milter, "550 5.7.1 Command rejected"));
 	    } else {
-		return ("550 5.7.1 Command rejected");
+		MILTER8_EVENT_BREAK("550 5.7.1 Command rejected");
 	    }
 
 	    /*
@@ -1091,10 +1089,10 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		milter8_close_stream(milter);
 #endif
 		milter->state = MILTER8_STAT_REJECT_CON;
-		return (milter8_def_reply(milter,
+		MILTER8_EVENT_BREAK(milter8_def_reply(milter,
 			"451 4.7.1 Service unavailable - try again later"));
 	    } else {
-		return ("451 4.7.1 Service unavailable - try again later");
+		MILTER8_EVENT_BREAK("451 4.7.1 Service unavailable - try again later");
 	    }
 
 	    /*
@@ -1109,7 +1107,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 	    milter8_close_stream(milter);
 #endif
 	    milter->state = MILTER8_STAT_REJECT_CON;
-	    return (milter8_def_reply(milter, "S"));
+	    MILTER8_EVENT_BREAK(milter8_def_reply(milter, "S"));
 #endif
 
 	    /*
@@ -1126,7 +1124,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 	    if (milter8_read_data(milter, data_size,
 				  MILTER8_DATA_BUFFER, milter->buf,
 				  MILTER8_DATA_END) != 0)
-		return (milter->def_reply);
+		MILTER8_EVENT_BREAK(milter->def_reply);
 	    if ((STR(milter->buf)[0] != '4' && STR(milter->buf)[0] != '5')
 		|| !ISDIGIT(STR(milter->buf)[1])
 		|| !ISDIGIT(STR(milter->buf)[2])
@@ -1135,7 +1133,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		msg_warn("milter %s: malformed reply: %s",
 			 milter->m.name, STR(milter->buf));
 		milter8_conf_error(milter);
-		return (milter->def_reply);
+		MILTER8_EVENT_BREAK(milter->def_reply);
 	    }
 	    if ((rp = cp = strchr(STR(milter->buf), '%')) != 0) {
 		for (;;) {
@@ -1151,9 +1149,9 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		milter8_close_stream(milter);
 #endif
 		milter->state = MILTER8_STAT_REJECT_CON;
-		return (milter8_def_reply(milter, STR(milter->buf)));
+		MILTER8_EVENT_BREAK(milter8_def_reply(milter, STR(milter->buf)));
 	    } else {
-		return (STR(milter->buf));
+		MILTER8_EVENT_BREAK(STR(milter->buf));
 	    }
 
 	    /*
@@ -1167,8 +1165,8 @@ static const char *milter8_event(MILTER8 *milter, int event,
 	    if (milter8_read_data(milter, data_size,
 				  MILTER8_DATA_BUFFER, milter->buf,
 				  MILTER8_DATA_END) != 0)
-		return (milter->def_reply);
-	    return ("H");
+		MILTER8_EVENT_BREAK(milter->def_reply);
+	    MILTER8_EVENT_BREAK("H");
 #endif
 
 	    /*
@@ -1189,8 +1187,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 					  MILTER8_DATA_STRING, milter->buf,
 					  MILTER8_DATA_STRING, milter->body,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
-		    parent = milter->m.parent;
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    /* XXX Sendmail 8 compatibility. */
 		    if (index == 0)
 			index = 1;
@@ -1198,13 +1195,13 @@ static const char *milter8_event(MILTER8 *milter, int event,
 			msg_warn("milter %s: bad change header index: %ld",
 				 milter->m.name, (long) index);
 			milter8_conf_error(milter);
-			return (milter->def_reply);
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    }
 		    if (LEN(milter->buf) == 0) {
 			msg_warn("milter %s: null change header name",
 				 milter->m.name);
 			milter8_conf_error(milter);
-			return (milter->def_reply);
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    }
 		    if (STR(milter->body)[0])
 			edit_resp = parent->upd_header(parent->chg_context,
@@ -1216,7 +1213,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 						       (ssize_t) index,
 						       STR(milter->buf));
 		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(edit_resp);
 		    continue;
 #endif
 
@@ -1228,13 +1225,12 @@ static const char *milter8_event(MILTER8 *milter, int event,
 					  MILTER8_DATA_STRING, milter->buf,
 					  MILTER8_DATA_STRING, milter->body,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
-		    parent = milter->m.parent;
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    edit_resp = parent->add_header(parent->chg_context,
 						   STR(milter->buf),
 						   STR(milter->body));
 		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(edit_resp);
 		    continue;
 
 		    /*
@@ -1250,20 +1246,19 @@ static const char *milter8_event(MILTER8 *milter, int event,
 					  MILTER8_DATA_STRING, milter->buf,
 					  MILTER8_DATA_STRING, milter->body,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    if ((ssize_t) index + 1 < 1) {
 			msg_warn("milter %s: bad insert header index: %ld",
 				 milter->m.name, (long) index);
 			milter8_conf_error(milter);
-			return (milter->def_reply);
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    }
-		    parent = milter->m.parent;
 		    edit_resp = parent->ins_header(parent->chg_context,
 						   (ssize_t) index + 1,
 						   STR(milter->buf),
 						   STR(milter->body));
 		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(edit_resp);
 		    continue;
 #endif
 
@@ -1274,12 +1269,11 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		    if (milter8_read_data(milter, data_size,
 					  MILTER8_DATA_STRING, milter->buf,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
-		    parent = milter->m.parent;
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    edit_resp = parent->add_rcpt(parent->chg_context,
 						 STR(milter->buf));
 		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(edit_resp);
 		    continue;
 
 		    /*
@@ -1289,31 +1283,47 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		    if (milter8_read_data(milter, data_size,
 					  MILTER8_DATA_STRING, milter->buf,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
-		    parent = milter->m.parent;
+			MILTER8_EVENT_BREAK(milter->def_reply);
 		    edit_resp = parent->del_rcpt(parent->chg_context,
 						 STR(milter->buf));
 		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(edit_resp);
 		    continue;
 
 		    /*
 		     * Modification request: replace the message body, and
 		     * update the message size.
 		     */
-#if 0
 		case SMFIR_REPLBODY:
 		    if (milter8_read_data(milter, data_size,
 					  MILTER8_DATA_BUFFER, milter->body,
 					  MILTER8_DATA_END) != 0)
-			return (milter->def_reply);
-		    parent = milter->m.parent;
-		    edit_resp = parent->repl_body(parent->chg_context,
-						  milter->body);
-		    if (edit_resp)
-			return (edit_resp);
+			MILTER8_EVENT_BREAK(milter->def_reply);
+		    /* Start body replacement. */
+		    if (body_line_buf == 0) {
+			body_line_buf = vstring_alloc(var_line_limit);
+			edit_resp = parent->repl_body(parent->chg_context,
+						      MILTER_BODY_START,
+						      (VSTRING *) 0);
+		    }
+		    /* Extract lines from the on-the-wire CRLF format. */
+		    for (cp = STR(milter->body); edit_resp == 0
+			 && cp < vstring_end(milter->body); cp++) {
+			ch = *(unsigned char *) cp;
+			if (ch == '\n') {
+			    if (LEN(body_line_buf) > 0
+				&& vstring_end(body_line_buf)[-1] == '\r')
+				vstring_truncate(body_line_buf,
+						 LEN(body_line_buf) - 1);
+			    edit_resp = parent->repl_body(parent->chg_context,
+							  MILTER_BODY_LINE,
+							  body_line_buf);
+			    VSTRING_RESET(body_line_buf);
+			} else {
+			    VSTRING_ADDCH(body_line_buf, ch);
+			}
+		    }
 		    continue;
-#endif
 		}
 	    }
 	    msg_warn("milter %s: unexpected filter response %s after event %s",
@@ -1323,7 +1333,7 @@ static const char *milter8_event(MILTER8 *milter, int event,
 		     (smfic_name = str_name_code(smfic_table, event)) != 0 ?
 		     smfic_name : "(unknown MTA event)");
 	    milter8_comm_error(milter);
-	    return (milter->def_reply);
+	    MILTER8_EVENT_BREAK(milter->def_reply);
 	}
 
 	/*
@@ -1334,8 +1344,40 @@ static const char *milter8_event(MILTER8 *milter, int event,
 	milter->m.name, (smfir_name = str_name_code(smfir_table, cmd)) != 0 ?
 		 smfir_name : "unknown", (long) data_len);
 	milter8_comm_error(milter);
-	return (milter->def_reply);
+	MILTER8_EVENT_BREAK(milter->def_reply);
     }
+
+    /*
+     * Finish message body replacement.
+     */
+    if (body_line_buf != 0) {
+	if (edit_resp == 0) {
+	    /* In case the last body replacement line didn't end in CRLF. */
+	    if (LEN(body_line_buf) > 0)
+		edit_resp = parent->repl_body(parent->chg_context,
+					      MILTER_BODY_LINE,
+					      body_line_buf);
+	    if (edit_resp == 0)
+		edit_resp = parent->repl_body(parent->chg_context,
+					      MILTER_BODY_END,
+					      (VSTRING *) 0);
+	}
+	vstring_free(body_line_buf);
+
+	/*
+	 * Override a non-reject/discard result value after body replacement
+	 * failure.
+	 * 
+	 * XXX Some cleanup clients ask the cleanup server to bounce mail for
+	 * them. In that case we must override a hard reject retval result
+	 * after queue file update failure. This is not a big problem; the
+	 * odds are small that a Milter application sends a hard reject after
+	 * replacing the message body.
+	 */
+	if (edit_resp && (retval == 0 || strchr("DS4", retval[0]) == 0))
+	    retval = edit_resp;
+    }
+    return (retval);
 }
 
 /* milter8_connect - connect to filter */
@@ -1351,7 +1393,7 @@ static void milter8_connect(MILTER8 *milter)
     int     fd;
     const UINT32_TYPE my_actions = (SMFIF_ADDHDRS | SMFIF_ADDRCPT
 				    | SMFIF_DELRCPT | SMFIF_CHGHDRS
-    /* Not yet: | SMFIF_CHGBODY */
+				    | SMFIF_CHGBODY
 #ifdef SMFIF_QUARANTINE
 				    | SMFIF_QUARANTINE
 #endif
@@ -1670,6 +1712,7 @@ static const char *milter8_helo_event(MILTER *m, const char *helo_name,
 	return (milter->def_reply);
     case MILTER8_STAT_ENVELOPE:
     case MILTER8_STAT_ACCEPT_MSG:
+	/* With HELO after MAIL, smtpd(8) calls milter8_abort() next. */
 	if (msg_verbose)
 	    msg_info("%s: milter %s: helo %s",
 		     myname, milter->m.name, helo_name);
