@@ -29,10 +29,9 @@
 /*
 /*	To prevent Postfix from sending multiple recipients per delivery
 /*	request, specify
-/* .sp
-/* .nf
-/*	    \fItransport\fB_destination_recipient_limit = 1\fR
-/* .fi
+/*
+/* .ti +4
+/*	\fItransport\fB_destination_recipient_limit = 1\fR
 /*
 /*	in the Postfix \fBmain.cf\fR file, where \fItransport\fR
 /*	is the name in the first column of the Postfix \fBmaster.cf\fR
@@ -119,8 +118,8 @@
 /*	by, for example, \fBUUCP\fR software.
 /* .RE
 /* .IP "\fBnull_sender\fR=\fIreplacement\fR (default: MAILER-DAEMON)"
-/*	Replace the null sender address (typically used for delivery
-/*	status notifications) with the specified text
+/*	Replace the null sender address, which is typically used
+/*	for delivery status notifications, with the specified text
 /*	when expanding the \fB$sender\fR command-line macro, and
 /*	when generating a From_ or Return-Path: message header.
 /*
@@ -136,19 +135,17 @@
 /*	Caution: a null sender address is easily mis-parsed by
 /*	naive software. For example, when the \fBpipe\fR(8) daemon
 /*	executes a command such as:
-/* .sp
-/* .nf
-/*	    command -f$sender -- $recipient (\fIbad\fR)
-/* .fi
-/* .IP
+/*
+/* .ti +4
+/*	command -f$sender -- $recipient (\fIbad\fR)
+/*
 /*	the command will mis-parse the -f option value when the
 /*	sender address is a null string.  For correct parsing,
 /*	specify \fB$sender\fR as an argument by itself:
-/* .sp
-/* .nf
-/*	    command -f $sender -- $recipient (\fIgood\fR)
-/* .fi
-/* .IP
+/*
+/* .ti +4
+/*	command -f $sender -- $recipient (\fIgood\fR)
+/*
 /*	This feature is available with Postfix 2.3 and later.
 /* .IP "\fBsize\fR=\fIsize_limit\fR (optional)"
 /*	Messages greater in size than this limit (in bytes) will
@@ -332,11 +329,11 @@
 /*	The UNIX system account that owns the Postfix queue and most Postfix
 /*	daemon processes.
 /* .IP "\fBmax_idle (100s)\fR"
-/*	The maximum amount of time that an idle Postfix daemon process waits
-/*	for an incoming connection before terminating voluntarily.
+/*	The maximum amount of time that an idle Postfix daemon process
+/*	waits for the next service request before exiting.
 /* .IP "\fBmax_use (100)\fR"
-/*	The maximal number of incoming connections that a Postfix daemon
-/*	process will service before terminating voluntarily.
+/*	The maximal number of connection requests before a Postfix daemon
+/*	process terminates.
 /* .IP "\fBprocess_id (read-only)\fR"
 /*	The process ID of a Postfix command or daemon process.
 /* .IP "\fBprocess_name (read-only)\fR"
@@ -403,7 +400,6 @@
 #include <recipient_list.h>
 #include <deliver_request.h>
 #include <mail_params.h>
-#include <mail_version.h>
 #include <mail_conf.h>
 #include <bounce.h>
 #include <defer.h>
@@ -848,7 +844,7 @@ static void get_service_attr(PIPE_ATTR *attr, char **argv)
 	/*
 	 * null_sender=string
 	 */
-	else if (strncasecmp("null_sender=", *argv, sizeof("null_sender=") - 1) == 0) {
+	else if (strncasecmp("null_sender=", *argv, sizeof("eol=") - 1) == 0) {
 	    vstring_strcpy(attr->null_sender, *argv + sizeof("null_sender=") - 1);
 	}
 
@@ -1200,8 +1196,6 @@ static void pre_init(char *unused_name, char **unused_argv)
     flush_init();
 }
 
-MAIL_VERSION_STAMP_DECLARE;
-
 /* main - pass control to the single-threaded skeleton */
 
 int     main(int argc, char **argv)
@@ -1211,11 +1205,6 @@ int     main(int argc, char **argv)
 	0,
     };
 
-    /*
-     * Fingerprint executables and core dumps.
-     */
-    MAIL_VERSION_STAMP_ALLOCATE;
-
     single_server_main(argc, argv, pipe_service,
 		       MAIL_SERVER_TIME_TABLE, time_table,
 		       MAIL_SERVER_PRE_INIT, pre_init,
