@@ -390,8 +390,11 @@ int     var_proc_limit;
 bool    var_verp_bounce_off;
 int     var_qmgr_clog_warn_time;
 
-static QMGR_SCAN *qmgr_incoming;
-static QMGR_SCAN *qmgr_deferred;
+static QMGR_SCAN *qmgr_scans[2];
+
+#define QMGR_SCAN_IDX_INCOMING 0
+#define QMGR_SCAN_IDX_DEFERRED 1
+#define QMGR_SCAN_IDX_COUNT (sizeof(qmgr_scans) / sizeof(qmgr_scans[0]))
 
 /* qmgr_deferred_run_event - queue manager heartbeat */
 
@@ -402,7 +405,7 @@ static void qmgr_deferred_run_event(int unused_event, char *dummy)
      * This routine runs when it is time for another deferred queue scan.
      * Make sure this routine gets called again in the future.
      */
-    qmgr_scan_request(qmgr_deferred, QMGR_SCAN_START);
+    qmgr_scan_request(qmgr_scans[QMGR_SCAN_IDX_DEFERRED], QMGR_SCAN_START);
     event_request_timer(qmgr_deferred_run_event, dummy, var_queue_run_delay);
 }
 
@@ -462,19 +465,22 @@ static void qmgr_trigger_event(char *buf, int len,
      * requested, the request takes effect immediately.
      */
     if (incoming_flag != 0)
-	qmgr_scan_request(qmgr_incoming, incoming_flag);
+	qmgr_scan_request(qmgr_scans[QMGR_SCAN_IDX_INCOMING], incoming_flag);
     if (deferred_flag != 0)
-	qmgr_scan_request(qmgr_deferred, deferred_flag);
+	qmgr_scan_request(qmgr_scans[QMGR_SCAN_IDX_DEFERRED], deferred_flag);
 }
 
 /* qmgr_loop - queue manager main loop */
 
 static int qmgr_loop(char *unused_name, char **unused_argv)
 {
-    char   *in_path = 0;
-    char   *df_path = 0;
+    char   *path;
     int     token_count;
-    int     in_feed = 0;
+    int     feed = 0;
+    int     scan_idx;			/* Priority order scan index */
+    static int first_scan_idx = QMGR_SCAN_IDX_INCOMING;
+    int     last_scan_idx = QMGR_SCAN_IDX_COUNT - 1;
+    int     delay;
 
     /*
      * This routine runs as part of the event handling loop, after the event
@@ -495,15 +501,32 @@ static int qmgr_loop(char *unused_name, char **unused_argv)
 
     /*
      * Let some new blood into the active queue when the queue size is
-     * smaller than some configurable limit. When the system is under heavy
-     * load, favor new mail over old mail.
+     * smaller than some configurable limit.
+     * 
+     * We import one message per interrupt, to optimally tune the input count
+     * for the number of delivery agent protocol wait states, as explained in
+     * qmgr_transport.c.
      */
-    if (qmgr_message_count < var_qmgr_active_limit)
-	if ((in_path = qmgr_scan_next(qmgr_incoming)) != 0)
-	    in_feed = qmgr_active_feed(qmgr_incoming, in_path);
-    if (qmgr_message_count < var_qmgr_active_limit)
-	if ((df_path = qmgr_scan_next(qmgr_deferred)) != 0)
-	    qmgr_active_feed(qmgr_deferred, df_path);
+    delay = WAIT_FOR_EVENT;
+    for (scan_idx = 0; qmgr_message_count < var_qmgr_active_limit
+	 && scan_idx < QMGR_SCAN_IDX_COUNT; ++scan_idx) {
+	last_scan_idx = (scan_idx + first_scan_idx) % QMGR_SCAN_IDX_COUNT;
+	if ((path = qmgr_scan_next(qmgr_scans[last_scan_idx])) != 0) {
+	    delay = DONT_WAIT;
+	    if ((feed = qmgr_active_feed(qmgr_scans[last_scan_idx], path)) != 0)
+		break;
+	}
+    }
+
+    /*
+     * Round-robin the queue scans. When the active queue becomes full,
+     * prefer new mail over deferred mail.
+     */
+    if (qmgr_message_count < var_qmgr_active_limit) {
+	first_scan_idx = (last_scan_idx + 1) % QMGR_SCAN_IDX_COUNT;
+    } else if (first_scan_idx != QMGR_SCAN_IDX_INCOMING) {
+	first_scan_idx = QMGR_SCAN_IDX_INCOMING;
+    }
 
     /*
      * Global flow control. If enabled, slow down receiving processes that
@@ -512,17 +535,15 @@ static int qmgr_loop(char *unused_name, char **unused_argv)
     if (var_in_flow_delay > 0) {
 	token_count = mail_flow_count();
 	if (token_count < var_proc_limit) {
-	    if (in_feed != 0)
+	    if (feed != 0 && last_scan_idx == QMGR_SCAN_IDX_INCOMING)
 		mail_flow_put(1);
-	    else if (qmgr_incoming->handle == 0)
+	    else if (qmgr_scans[QMGR_SCAN_IDX_INCOMING]->handle == 0)
 		mail_flow_put(var_proc_limit - token_count);
 	} else if (token_count > var_proc_limit) {
 	    mail_flow_get(token_count - var_proc_limit);
 	}
     }
-    if (in_path || df_path)
-	return (DONT_WAIT);
-    return (WAIT_FOR_EVENT);
+    return (delay);
 }
 
 /* pre_accept - see if tables have changed */
@@ -588,9 +609,9 @@ static void qmgr_post_init(char *name, char **unused_argv)
     var_use_limit = 0;
     var_idle_limit = 0;
     qmgr_move(MAIL_QUEUE_ACTIVE, MAIL_QUEUE_INCOMING, event_time());
-    qmgr_incoming = qmgr_scan_create(MAIL_QUEUE_INCOMING);
-    qmgr_deferred = qmgr_scan_create(MAIL_QUEUE_DEFERRED);
-    qmgr_scan_request(qmgr_incoming, QMGR_SCAN_START);
+    qmgr_scans[QMGR_SCAN_IDX_INCOMING] = qmgr_scan_create(MAIL_QUEUE_INCOMING);
+    qmgr_scans[QMGR_SCAN_IDX_DEFERRED] = qmgr_scan_create(MAIL_QUEUE_DEFERRED);
+    qmgr_scan_request(qmgr_scans[QMGR_SCAN_IDX_INCOMING], QMGR_SCAN_START);
     qmgr_deferred_run_event(0, (char *) 0);
 }
 
