@@ -178,8 +178,7 @@ static int dict_pcre_expand(int type, VSTRING *buf, char *ptr)
 				 ctxt->matches, n, &pp);
 	if (ret < 0) {
 	    if (ret == PCRE_ERROR_NOSUBSTRING)
-		msg_fatal("regexp %s, line %d: replace index out of range",
-			  dict_pcre->dict.name, match_rule->rule.lineno);
+		return (MAC_PARSE_UNDEF);
 	    else
 		msg_fatal("regexp %s, line %d: pcre_get_substring error: %d",
 			dict_pcre->dict.name, match_rule->rule.lineno, ret);
@@ -265,7 +264,9 @@ static const char *dict_pcre_lookup(DICT *dict, const char *lookup_string)
     /*
      * Optionally fold the key.
      */
-    if (dict->fold_buf) {
+    if (dict->flags & DICT_FLAG_FOLD_MUL) {
+	if (dict->fold_buf == 0)
+	    dict->fold_buf = vstring_alloc(10);
 	vstring_strcpy(dict->fold_buf, lookup_string);
 	lookup_string = lowercase(vstring_str(dict->fold_buf));
     }
@@ -591,6 +592,7 @@ static DICT_PCRE_RULE *dict_pcre_parse_rule(const char *mapname, int lineno,
 					            int dict_flags)
 {
     char   *p;
+    int     actual_sub;
 
     p = line;
 
@@ -662,6 +664,23 @@ static DICT_PCRE_RULE *dict_pcre_parse_rule(const char *mapname, int lineno,
 	 */
 	if (dict_pcre_compile(mapname, lineno, &regexp, &engine) == 0)
 	    CREATE_MATCHOP_ERROR_RETURN(0);
+#ifdef PCRE_INFO_CAPTURECOUNT
+	if (pcre_fullinfo(engine.pattern, engine.hints,
+			  PCRE_INFO_CAPTURECOUNT,
+			  (void *) &actual_sub) != 0)
+	    msg_panic("pcre map %s, line %d: pcre_fullinfo failed",
+		      mapname, lineno);
+	if (prescan_context.max_sub > actual_sub) {
+	    msg_warn("regexp map %s, line %d: out of range replacement index \"%d\": "
+		     "skipping this rule", mapname, lineno,
+		     (int) prescan_context.max_sub);
+	    if (engine.pattern)
+		myfree((char *) engine.pattern);
+	    if (engine.hints)
+		myfree((char *) engine.hints);
+	    CREATE_MATCHOP_ERROR_RETURN(0);
+	}
+#endif
 
 	/*
 	 * Save the result.
